<?php
/*
 * Plugin Name: Dev Blocks
 * Plugin URI: https://devhuset.no
 * Description: A plugin to add custom blocks to the Elementor editor.
 * Version: 1.0.53
 * Requires at least: 5.4
 * Requires PHP: 8.1
 * Author: Devhuset
 * Author URI: https://devhuset.no
 * License: GPL v2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: dev-blocks
 */

if (!defined('DEV_CURRENT_VERSION')) {
    define('DEV_CURRENT_VERSION', '1.0.53');
}

if (!defined('DEV_PLUGIN_DIR')) {
    define('DEV_PLUGIN_DIR', WP_PLUGIN_DIR . '/' . plugin_basename(__DIR__));
}

if (!defined('DEV_RELATIVE_PLUGIN_DIR')) {
    define('DEV_RELATIVE_PLUGIN_DIR', plugin_basename(__DIR__));
}

if (!defined('DEV_PLUGIN_FILE')) {
    define('DEV_PLUGIN_FILE', WP_PLUGIN_DIR . '/' . plugin_basename(__FILE__));
}

if (!defined('DEV_RELATIVE_PLUGIN_FILE')) {
    define('DEV_RELATIVE_PLUGIN_FILE', plugin_basename(__FILE__));
}

require DEV_PLUGIN_DIR . '/update.php';

/**
 * Register Elementor Dev Blocks.
 */
add_action('elementor/widgets/register', 'register_new_widgets');

function register_new_widgets($widgets_manager)
{
    require_once(DEV_PLUGIN_DIR . '/blocks/event-grid/block.php');
    require_once(DEV_PLUGIN_DIR . '/blocks/event-calendar/block.php');

    $widgets_manager->register(new \Elementor_Event_Grid_Widget());
    $widgets_manager->register(new \Elementor_Event_Calendar_Widget());
}

/**
 * Register scripts and styles for Dev Blocks.
 */
add_action('wp_enqueue_scripts', 'dev_blocks_dependencies');

function dev_blocks_dependencies()
{
    /* Scripts */
    wp_enqueue_script('event-calendar-script', plugins_url('blocks/event-calendar/script.js', __FILE__), array(), null, true);
    wp_localize_script(
        'event-calendar-script',
        'ajax_object',
        array(
            'ajaxurl' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('calendar_nonce')
        )
    );

    /* Styles */
    wp_enqueue_style('event-grid-style', plugins_url('blocks/event-grid/style.css', __FILE__));
    wp_enqueue_style('event-calendar-style', plugins_url('blocks/event-calendar/style.css', __FILE__));

}

function build_calendar($year, $month)
{
    $start_date = date("Y-m-01 00:00:00", strtotime("$year-$month-01"));
    $end_date = date("Y-m-t 23:59:59", strtotime("$year-$month-01"));
    $args = [
        'post_type' => 'events',
        'posts_per_page' => -1,
        'meta_query' => [
            [
                'key' => 'event_start',
                'value' => [$start_date, $end_date],
                'compare' => 'BETWEEN',
                'type' => 'DATETIME'
            ]
        ],
        'orderby' => 'meta_value',
        'order' => 'ASC'
    ];

    $the_query = new WP_Query($args);

    ob_start();
    // Pass all necessary variables to the template
    $template_vars = [
        'year' => $year,
        'month' => $month,
        'the_query' => $the_query,
        'prev_year' => $month == 1 ? $year - 1 : $year,
        'prev_month' => $month == 1 ? 12 : $month - 1,
        'next_year' => $month == 12 ? $year + 1 : $year,
        'next_month' => $month == 12 ? 1 : $month + 1,
        'current_year' => date('Y'),
        'current_month' => date('m')
    ];
    extract($template_vars);
    include(DEV_PLUGIN_DIR . '/blocks/event-calendar/calendar-template.php');
    wp_reset_query();
    $calendar_html = ob_get_clean();
    return $calendar_html;
}

function handle_calendar_ajax()
{
    require_once(DEV_PLUGIN_DIR . '/blocks/event-calendar/block.php');

    if (!isset($_POST['nonce']) || !check_ajax_referer('calendar_nonce', 'nonce', false)) {
        wp_send_json_error('Nonce is incorrect or not set', 401);
        wp_die();
    }

    $year = $_POST['year'] ?? date('Y');
    $month = $_POST['month'] ?? date('m');

    $_SESSION['year'] = $year;
    $_SESSION['month'] = $month;

    if (!class_exists('Elementor_Event_Calendar_Widget')) {
        wp_send_json_error('Widget class not found', 500);
        wp_die();
    }

    echo get_or_build_calendar($year, $month);
    wp_die();
}

function get_or_build_calendar($year, $month)
{
    $transient_key = "calendar_data_{$year}_{$month}";
    $calendar_html = get_transient($transient_key);
    if (false === $calendar_html) {
        // No valid cache found, build the calendar
        $calendar_html = build_calendar($year, $month);
        // Cache the HTML for 1 day
        set_transient($transient_key, $calendar_html, 1 * DAY_IN_SECONDS);
    }
    return $calendar_html;
}


function handle_ajax()
{
    if (defined('DOING_AJAX') && DOING_AJAX) {
        add_action('wp_ajax_calendar_nav', 'handle_calendar_ajax');  // For logged-in users
        add_action('wp_ajax_nopriv_calendar_nav', 'handle_calendar_ajax');  // For logged-out users if necessary
    }
}
add_action('plugins_loaded', 'handle_ajax');

function clear_calendar_cache($post_id)
{
    // Check if the post is really an 'event'
    if (get_post_type($post_id) != 'events') {
        return;
    }

    // Clear transients related to the calendar
    global $wpdb;

    // delete all transients that start with 'calendar_data_'
    $wpdb->query("DELETE FROM $wpdb->options WHERE option_name LIKE '_transient_calendar_data_%' OR option_name LIKE '_transient_timeout_calendar_data_%'");
}

// Hooks to clear the calendar cache on saving or deleting events
add_action('save_post_events', 'clear_calendar_cache');
add_action('before_delete_post', 'clear_calendar_cache');
